import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:ir_datetime_picker/src/helpers/date.dart';
import 'package:ir_datetime_picker/src/helpers/print.dart';
import 'package:ir_datetime_picker/src/helpers/responsive.dart';
import 'package:shamsi_date/shamsi_date.dart';

/// * [IRGregorianDatePickerOnSelected] is a callback function that will call when user change cupertino pickers.

typedef IRGregorianDatePickerOnSelected = void Function(
    Gregorian gregorianDate);

/// * You can use [IRGregorianDatePicker] to design your own date pickers.

class IRGregorianDatePicker extends StatefulWidget {
  final Gregorian? initialDate;
  final int? minYear;
  final int? maxYear;
  final bool visibleTodayButton;
  final String todayButtonText;
  final BoxConstraints? constraints;
  final IRGregorianDatePickerOnSelected onSelected;
  final TextStyle? textStyle;
  final double diameterRatio;
  final double magnification;
  final double offAxisFraction;
  final double squeeze;

  const IRGregorianDatePicker({
    super.key,
    this.initialDate,
    this.minYear,
    this.maxYear,
    this.visibleTodayButton = true,
    required this.todayButtonText,
    this.constraints,
    required this.onSelected,
    this.textStyle,
    this.diameterRatio = 1.0,
    this.magnification = 1.3,
    this.offAxisFraction = 0.0,
    this.squeeze = 1.3,
  });

  @override
  State<IRGregorianDatePicker> createState() => _IRGregorianDatePickerState();
}

class _IRGregorianDatePickerState extends State<IRGregorianDatePicker> {
  late Gregorian _initialDate;
  late bool _refreshCupertinoPickers;
  int _selectedYear = 2020;
  int _selectedMonth = 1;
  int _selectedDay = 1;
  List<int> _years = [];
  final List<String> _months = IRGregorianDateHelper.months;
  List<int> _days = [];

  @override
  void initState() {
    super.initState();
    _initialDate = widget.initialDate ?? Gregorian.now();
    _refreshCupertinoPickers = false;
    _selectedYear = _initialDate.year;
    _selectedMonth = _initialDate.month;
    _selectedDay = _initialDate.day;
    _years = _yearsList(widget.minYear ?? (_initialDate.year - 50),
        widget.maxYear ?? (_initialDate.year + 50));
    _days = _daysList(_getSelectedGregorianDate().monthLength);
  }

  @override
  Widget build(BuildContext context) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _refreshCupertinoPickers = false;
    });
    BoxConstraints cupertinoPickersConstraints = BoxConstraints.loose(
      Size(100.0.percentOfWidth(context), 30.0.percentOfHeight(context)),
    );
    Widget cupertinoPickers = Directionality(
      textDirection: TextDirection.ltr,
      child: ConstrainedBox(
        constraints: widget.constraints ?? cupertinoPickersConstraints,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _cupertinoPicker(
              context: context,
              list: _years,
              initialItem: _years.indexOf(_selectedYear),
              onSelectedItemChanged: (selectedIndex) {
                setState(() {
                  _selectedYear = _years[selectedIndex];
                  int monthLength = IRGregorianDateHelper.getMonthLength(
                      year: _selectedYear, month: _selectedMonth);
                  _days = List<int>.generate(monthLength, (index) => index + 1);
                  if (_selectedDay > monthLength) {
                    _selectedDay = monthLength;
                  }
                });
                widget.onSelected(_getSelectedGregorianDate());
              },
            ),
            _cupertinoPicker(
              context: context,
              list: _months,
              initialItem: _months.indexOf(IRGregorianDateHelper.getMonthName(
                  monthNumber: _selectedMonth)),
              onSelectedItemChanged: (selectedIndex) {
                setState(() {
                  _selectedMonth = IRGregorianDateHelper.getMonthNumber(
                      monthName: _months[selectedIndex]);
                  int monthLength = IRGregorianDateHelper.getMonthLength(
                      year: _selectedYear, month: _selectedMonth);
                  _days = List<int>.generate(monthLength, (index) => index + 1);
                  if (_selectedDay > monthLength) {
                    _selectedDay = monthLength;
                  }
                });
                widget.onSelected(_getSelectedGregorianDate());
              },
            ),
            _cupertinoPicker(
              context: context,
              list: _days,
              initialItem: _days.indexOf(_selectedDay),
              onSelectedItemChanged: (selectedIndex) {
                _selectedDay = _days[selectedIndex];
                widget.onSelected(_getSelectedGregorianDate());
              },
            ),
          ],
        ),
      ),
    );
    Widget todayButton = Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        SizedBox(height: 1.0.percentOfHeight(context)),
        Padding(
          padding:
              EdgeInsets.symmetric(horizontal: 10.0.percentOfWidth(context)),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.start,
            mainAxisSize: MainAxisSize.max,
            children: [
              TextButton.icon(
                icon: Icon(Icons.info,
                    size: 6.5.percentOfWidth(context),
                    color: widget.textStyle?.color ??
                        Theme.of(context).textTheme.titleMedium?.color),
                style: TextButton.styleFrom(
                  padding: EdgeInsets.all(2.0.percentOfWidth(context)),
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(12.0)),
                ),
                label: Text(widget.todayButtonText,
                    style: (widget.textStyle ??
                            Theme.of(context).textTheme.titleMedium)
                        ?.copyWith(
                            fontSize: 14.responsiveFont(context),
                            fontWeight: FontWeight.w600)),
                onPressed: () {
                  setState(() {
                    _refreshCupertinoPickers = true;
                    Gregorian now = Gregorian.now();
                    _selectedYear = now.year;
                    _selectedMonth = now.month;
                    _selectedDay = now.day;
                  });
                  widget.onSelected(_getSelectedGregorianDate());
                },
              ),
            ],
          ),
        ),
      ],
    );
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        cupertinoPickers,
        Visibility(
          visible: widget.visibleTodayButton,
          child: todayButton,
        ),
      ],
    );
  }

  Widget _cupertinoPicker(
      {required BuildContext context,
      required List list,
      required int initialItem,
      required ValueChanged<int> onSelectedItemChanged}) {
    mPrint(initialItem);
    BoxConstraints cupertinoPickerConstraints = BoxConstraints.loose(
      Size(30.0.percentOfWidth(context), double.infinity),
    );
    return ConstrainedBox(
      constraints: cupertinoPickerConstraints,
      child: CupertinoPicker(
        key: _refreshCupertinoPickers ? UniqueKey() : null,
        scrollController: FixedExtentScrollController(initialItem: initialItem),
        itemExtent: 8.5.percentOfWidth(context),
        diameterRatio: widget.diameterRatio,
        magnification: widget.magnification,
        offAxisFraction: widget.offAxisFraction,
        squeeze: widget.squeeze,
        selectionOverlay: Container(
          decoration: BoxDecoration(
            border: Border(
              top: BorderSide(
                  color: widget.textStyle?.color?.withOpacity(0.35) ??
                      Colors.grey.shade400,
                  width: 0.5),
              bottom: BorderSide(
                  color: widget.textStyle?.color?.withOpacity(0.35) ??
                      Colors.grey.shade400,
                  width: 0.5),
            ),
          ),
        ),
        onSelectedItemChanged: onSelectedItemChanged,
        children: list.map<Widget>(
          (element) {
            return Center(
              child: Text(
                element.toString(),
                style: Theme.of(context).textTheme.titleMedium?.copyWith(
                      color: widget.textStyle?.color,
                      fontSize: widget.textStyle?.fontSize ??
                          16.5.responsiveFont(context),
                      fontWeight: widget.textStyle?.fontWeight,
                    ),
              ),
            );
          },
        ).toList(),
      ),
    );
  }

  List<int> _yearsList(int minYear, int maxYear) {
    List<int> years = [];
    for (int i = minYear; i <= maxYear; i++) {
      years.add(i);
    }
    return years;
  }

  List<int> _daysList(int monthLength) {
    return List<int>.generate(monthLength, (index) => index + 1);
  }

  Gregorian _getSelectedGregorianDate() {
    return Gregorian(_selectedYear, _selectedMonth, _selectedDay);
  }
}
